#!/usr/bin/python
# V01.00 by negg (negg@aspekt1.net) see http://www.aspekt1.net/ms/dev/
# V01.01 by ZMI (phytonhacker@zmi.at) 2002-03-17: 
#               Made user vars for nicer configuration, changed
#               "makemap" to "makemap -r". Also changed the regex to
#               be used with "qpopper" of qualcomm.com, and to support usernames
#               that contain numbers, "-", "_" and ".".
# TODO: Sort "myLogIp" and filter out double IP's so that log looks nicer (if you
#         have a lot of pop logins from the same IP you get a lot of same entries...)
#
# TIP: I would suggest you set up your logging so that you log POP logins to an
# extra file. That way you can set "maxreadsize" to a small number, and the
# script has to read less entries and produces less load on your server if you
# set "sleepsecondsbetweenrun" to a low number for convenient use by your users.
# (5000 Bytes are 46 lines for me, a run every 3 secs produces no load)

#------------------
# User configurable variables:

# Read from which logfile
readfromlogfile="/var/log/popper"
# Remember to also change the regular expression for searching
# your log file! Search for "--RegEx--" to find the Line you should change

# How old could an entry be to be accepted?
secondsoldentry=90000

# Maximum Bytes we read back from the end of "readfromlogfile"
# You should set it so that the "secondsoldentry" seconds old entries
# can be read.
maxreadsize=5000

# Where to write the "makemap" file to?
dbfilename="/etc/mail/popip"

# Where should we write our log to?
poprelaydlog="/var/log/poprelayd"

# And where's the pidfile?
pidfile="/var/run/poprelayd.pid"

# how many seconds do we sleep between two runs?
# keep this small for fast reaction to pop logins, but be aware
# that it needs CPU time and prodcuses I/O
sleepsecondsbetweenrun=3

# imports
import os
import string
import re
import time


########## functions ############

def readLinesFromEnd(maxLen):

    fp=open(readfromlogfile)
        # seek to the end
    fp.seek(0,2)
        # seek back from the end
    fp.seek(-min(maxLen,fp.tell()),2)
        # Dump the first line
        # It might be chopped off
    lines=fp.readlines()[1:]
    return lines

########### main ################

pid = os.fork()
#DEBUG: comment out above line and set this line if you want to
#debug the output of your regex below!
#pid = 0
if pid == 0:
	# Child process (daemon)
	
	# flush variables for next turn
	myLogIp = ""
	myLogIpOld = ""
	
	while 1:
		
		logIp = []

		logFile = readLinesFromEnd(maxreadsize)
		tick = 0

		while tick < len(logFile):
			logLine = logFile[tick]
			logDate = logLine[0:15]
			# logfile-line-date younger than "secondsoldentry"?
			if time.time() - secondsoldentry <= time.mktime(time.strptime(time.strftime("%Y",time.localtime()) + " " + logDate,"%Y %b %d %X")):
				
				# is this line a successful pop-login? Adapt this regular expression to suit your needs
				# --RegEx--
				reMatch = re.search("^.*popper\[[0-9]+\]: Stats: [a-zA-Z0-9\-\_\.]+ [0-9]+ [0-9]+ [0-9]+ [0-9]+ (?P<ipadress>[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+).*",logLine)
				# Uncomment this for debugging your regular expression
				# (so you can see if it works!)
				# but remember to also uncomment the "pid=0" line above
				#print "\nlogline: " + `logLine` 
				#print "Match? " + `reMatch`
				if reMatch:
					ip = reMatch.group('ipadress')
					if ip[0:7] != "192.168":
						logIp = logIp + [ip + "	OK\n"]
						myLogIp = myLogIp + " " + ip
			tick = tick + 1
			
		# write popip
		d = open(dbfilename,"w")
		d.writelines(logIp)
		d.close()
		
		# make the map popip.db
		os.popen('makemap -r hash ' + dbfilename + '<' + dbfilename)
		os.close		
		
		# log action
		if myLogIp != myLogIpOld:
			popLog = open(poprelaydlog,"a")
			popLog.write(time.strftime("%Y %b %d %X",time.localtime()) + " : smtp ok for " + myLogIp + "\n")
			popLog.close()
		
		# flush variables for next turn
		myLogIpOld = myLogIp
		myLogIp = ""

		#wait some seconds before the next run
		time.sleep(sleepsecondsbetweenrun)
		
else:
	# Parent process:
	print "\npoprelayd started. PID = " + `pid`
	pidFile = open(pidfile,"w")
	pidFile.write(`pid`)
	pidFile.close()

